// PaintDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Graphics.h"
#include "PaintDlg.h"
#include ".\paintdlg.h"

#include "GraphicsDlg.h"
#include <atlimage.h>

// CPaintDlg dialog

IMPLEMENT_DYNAMIC(CPaintDlg, CDialog)
CPaintDlg::CPaintDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CPaintDlg::IDD, pParent)
{
}

CPaintDlg::~CPaintDlg()
{
}

void CPaintDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}


BEGIN_MESSAGE_MAP(CPaintDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_SIZE()
	ON_WM_MOVE()
END_MESSAGE_MAP()

// CPaintDlg message handlers

const COLORREF CPaintDlg::m_crColors[] = {
	RGB(0, 0, 0),		// Black - 
	RGB(0, 0, 255),		// Blue - 
	RGB(0, 255, 0),		// Green - 
	RGB(0, 255, 255),	// Cyan - 
	RGB(255, 0, 0),		// Red - 
	RGB(255, 0, 255),	// Magenta - 
	RGB(255, 255, 0),	// Yellow - 
	RGB(255, 255, 255)	// White - 
};


//    
void CPaintDlg::DrawLine(CPaintDC * pDC, int iColor)
{
	//    
	CPen pnSolidPen(PS_SOLID, 1, m_crColors[iColor]);
	CPen pnDashPen(PS_DASH, 1, m_crColors[iColor]);
	CPen pnDotPen(PS_DOT, 1, m_crColors[iColor]);
	CPen pnDashDotPen(PS_DASHDOT, 1, m_crColors[iColor]);
	CPen pnDashDotDotPen(PS_DASHDOTDOT, 1, m_crColors[iColor]);
	CPen pnNullPen(PS_NULL, 1, m_crColors[iColor]);
	CPen pnInsidePen(PS_INSIDEFRAME, 1, m_crColors[iColor]);
	CPen pnCombinationPen(PS_DOT|PS_GEOMETRIC|PS_ENDCAP_ROUND
					, 1, m_crColors[iColor]);

	//   
	CRect rRect;
	GetClientRect(rRect);
	rRect.NormalizeRect();

	//    
	CPoint ptStart;
	CPoint ptEnd;
	int iDisp = rRect.Height() / 8; //  
	CPen* pOldPen = NULL; //  
	//    
	ptStart.y = rRect.top + iDisp / 2;
	ptStart.x = rRect.left + iDisp / 2;
	ptEnd.y = ptStart.y;
	ptEnd.x = rRect.right - iDisp / 2;

	//      
	for(int count = 0; count < 8; count++){

		//     
		switch(count){
			case 0: //     
					//  ,    
					//      
				pOldPen = pDC->SelectObject(&pnSolidPen);
				break;
			case 1:
				pDC->SelectObject(&pnDashPen);
				break;
			case 2:
				pDC->SelectObject(&pnDotPen);
				break;
			case 3:
				pDC->SelectObject(&pnDashDotPen);
				break;
			case 4:
				pDC->SelectObject(&pnDashDotDotPen);
				break;
			case 5:
				pDC->SelectObject(&pnNullPen);
				break;
			case 6:
				pDC->SelectObject(&pnInsidePen);
				break;
			case 7:
				pDC->SelectObject(&pnCombinationPen);
		} // end of switch

		//  
		pDC->MoveTo(ptStart);
		pDC->LineTo(ptEnd);

		//     
		ptStart.y += iDisp;
		ptEnd.y = ptStart.y;

	} // end of for

	//     
	pDC->SelectObject(pOldPen);
}

void CPaintDlg::OnPaint()
{
	CPaintDC dc(this); //    

	//     
	CGraphicsDlg* pWnd = (CGraphicsDlg*) GetParent();

	if(!pWnd) return; //     - 

	if(pWnd->m_iTool == 0 //    Pen
	   && pWnd->m_iShape == 0) //     Line
		DrawLine(&dc, pWnd->m_iColor); //  

	if(pWnd->m_iTool != 2 //     Bitmap
	   && pWnd->m_iShape != 0) //     Line
		//  ,      
		DrawRegion(&dc, pWnd->m_iTool, pWnd->m_iShape, pWnd->m_iColor); 

	if(pWnd->m_iTool == 2) //   
		//  
		DrawImage(&dc, pWnd->m_iTool, pWnd->m_iShape, pWnd->m_iColor); 
}

//     GDI -   
void CPaintDlg::DrawRegion(CPaintDC * pDC, int iTool, int iShape, int iColor)
{
	//    
	CPen pnSolidPen(PS_SOLID, 1, m_crColors[iColor]);
	CPen pnDashPen(PS_DASH, 1, m_crColors[iColor]);
	CPen pnDotPen(PS_DOT, 1, m_crColors[iColor]);
	CPen pnDashDotPen(PS_DASHDOT, 1, m_crColors[iColor]);
	CPen pnDashDotDotPen(PS_DASHDOTDOT, 1, m_crColors[iColor]);
	CPen pnNullPen(PS_NULL, 1, m_crColors[iColor]);
	CPen pnInsidePen(PS_INSIDEFRAME, 1, m_crColors[iColor]);
	CPen pnCombinationPen(PS_DOT|PS_GEOMETRIC|PS_ENDCAP_ROUND
					, 1, m_crColors[iColor]);

	//    
	CBrush brSolidBrush(m_crColors[iColor]);
	CBrush brBDiagBrush(HS_BDIAGONAL, m_crColors[iColor]);
	CBrush brCrossBrush(HS_CROSS, m_crColors[iColor]);
	CBrush brDiagCrossBrush(HS_DIAGCROSS, m_crColors[iColor]);
	CBrush brFDiagBrush(HS_FDIAGONAL, m_crColors[iColor]);
	CBrush brHorizBrush(HS_HORIZONTAL, m_crColors[iColor]);
	CBrush brVertBrush(HS_VERTICAL, m_crColors[iColor]);
	CBrush brNullBrush(RGB(208, 208, 208)); // -  

	//   
	//   GDI  
	CRect rClientRect;
	GetClientRect(rClientRect);
	rClientRect.NormalizeRect();

	//  
	const int iDisp = 8;//        
	int iHeight = (rClientRect.Height() - 3 * iDisp) / 2; //     
	int iWidth = (rClientRect.Width() - 5 * iDisp) / 4; //      

	CRect rCurrentRect; //   
	CPen* pOldPen = NULL; //      
	CBrush* pOldBrush = NULL; //      

	//       
	//   
	for(int count = 0; count < 8; count ++){
		//     
		if(count == 0){ //   
			rCurrentRect.left = rClientRect.left + iDisp;//  X
			rCurrentRect.right = rCurrentRect.left + iWidth;//  X
			rCurrentRect.top = rClientRect.top + iDisp;//  Y
			rCurrentRect.bottom = rCurrentRect.top + iHeight;//  Y
		}
		else if(count == 4){ //   
			rCurrentRect.left = rClientRect.left + iDisp;//  X
			rCurrentRect.right = rCurrentRect.left + iWidth;//  X
			rCurrentRect.top = rClientRect.top + 2 * iDisp + iHeight;//  Y
			rCurrentRect.bottom = rCurrentRect.top + iHeight;//  Y
		}
		else{//   
			rCurrentRect.left = rCurrentRect.right + iDisp;//  X
			rCurrentRect.right = rCurrentRect.left + iWidth;//  X
		}

		//     
		switch(count){
			case 0: //     
				//      
				pOldPen = pDC->SelectObject(&pnSolidPen);
				pOldBrush = pDC->SelectObject(&brSolidBrush);
				break;
			case 1: //    
				pDC->SelectObject(&pnDashPen);
				pDC->SelectObject(&brBDiagBrush);
				break;
			case 2: //      
				pDC->SelectObject(&pnDotPen);
				pDC->SelectObject(&brCrossBrush);
				break;
			case 3: //      
				pDC->SelectObject(&pnDashDotPen);
				pDC->SelectObject(&brDiagCrossBrush);
				break;
			case 4: //       
				pDC->SelectObject(&pnDashDotDotPen);
				pDC->SelectObject(&brFDiagBrush);
				break;
			case 5: //  ,  , 
				pDC->SelectObject(&pnNullPen);
				pDC->SelectObject(&brHorizBrush);
				break;
			case 6: //   
				pDC->SelectObject(&pnInsidePen);
				pDC->SelectObject(&brVertBrush);
				break;
			case 7: //  
				pDC->SelectObject(&pnCombinationPen);
				pDC->SelectObject(&brNullBrush);
		} // end of switch

		//    Tool
		//  :    ( )?
		//      !!!
		switch(iTool){
			case 0: //   Tool = Pen -   
				//  
				pDC->SelectObject(&brNullBrush);
				break;
			case 1: //   Tool = Brush -   
				//  
				pDC->SelectObject(&pnNullPen);
				break;
			default:
				//        
				pDC->SelectObject(&pnNullPen);
				pDC->SelectObject(&brNullBrush);
		}

		//    Shape
		//  :    ( )?
		//        !!!
		switch(iShape){
			case 2: //   Shape = Square -  
				pDC->Rectangle(rCurrentRect);
				break;
			case 1: //   Shape = Circle -  
				pDC->Ellipse(rCurrentRect);
		}
	} // end of for

	//        
	pDC->SelectObject(pOldPen);
	pDC->SelectObject(pOldBrush);
}

void CPaintDlg::OnSize(UINT nType, int cx, int cy)
{
	CDialog::OnSize(nType, cx, cy);

	Invalidate(); //   
}

void CPaintDlg::OnMove(int x, int y)
{
	CDialog::OnMove(x, y);

	// TODO: Add your message handler code here

	Invalidate(); //   
}

void CPaintDlg::DrawImage(CPaintDC* dc, int iTool, int iShape, int iColor)
{
	//     
	//     ,   File Open
	CGraphicsDlg* pWnd = (CGraphicsDlg*) GetParent();
	if(!pWnd) return; //     - 

	//   
	CRect rClientRect;
	GetClientRect(rClientRect);
	rClientRect.NormalizeRect();

	//  
	const int iDisp = 7;//     
	int iWidth = rClientRect.Width() - 2 * iDisp; //   
	int iHeight = rClientRect.Height() - 2 * iDisp; //   

	//    , 
	//   ,   
	enum{Pen, Brush, Bitmap};
	enum{Line, Circle, Square};
	enum{Black, Blue, Green, Cyan
		,Red, Magenta, Yellow
		,White}; //   CImage ,   


/**************************************************************
    Shape==Line	-   
                    *.bmp 
                    API- Windows.

     Color     
     ,  
         .
**************************************************************/
	if(iShape == Line){
		HBITMAP hbitmap; //     
		int id;
		switch(iColor){
			case Black:   id = IDB_BITMAP1; break;
			case Blue: 	  id = IDB_BITMAP2; break;
			case Green:   id = IDB_BITMAP3; break;
			case Cyan:    id = IDB_BITMAP4; break;
			case Red:     id = IDB_BITMAP5; break;
			case Magenta: id = IDB_BITMAP6; break;
			case Yellow:  id = IDB_BITMAP7; break;
			case White:   id = IDB_BITMAP8;
		}
		
		//     
		hbitmap = ::LoadBitmap(::AfxGetInstanceHandle()
					,MAKEINTRESOURCE(id));

		//     
		HDC hMemDC = ::CreateCompatibleDC(NULL);

		//     dc 
		SelectObject(hMemDC, hbitmap);

		//   dc   dc 
		::StretchBlt(dc->m_hDC, iDisp, iDisp, iWidth, iHeight,// : dc 
			hMemDC, 0, 0, 400, 300, // : dc 
			SRCCOPY);

		//  dc    
		::DeleteDC(hMemDC);
		::DeleteObject(hbitmap);
	}


/**************************************************************
    Shape==Circle -    *.bmp, 
	                    File Open,
                       MFC- CBitmap.
	  Color   .
**************************************************************/
	if(iShape == Circle){

		//         
		HBITMAP hbitmap =
			(HBITMAP) ::LoadImage(AfxGetInstanceHandle(),
						         pWnd->strFileName, IMAGE_BITMAP, 0, 0,
								 LR_LOADFROMFILE | LR_CREATEDIBSECTION);

		if(hbitmap){//   
			//    CBitmap
			CBitmap m_Bitmap;
			//      
			m_Bitmap.Attach(hbitmap);
			//       CBitmap
			BITMAP bm;
			//     HBITMAP 
			//     m_Bitmap  BITMAP
			m_Bitmap.GetBitmap(&bm);
			//      
			//      CPaintDC* dc
			CDC dcMem;
			dcMem.CreateCompatibleDC(dc);
			//     dc 
			dcMem.SelectObject(m_Bitmap); //  ?
			//     
			//     
			dc->StretchBlt(iDisp, iDisp, iWidth, iHeight,// : dc 
				&dcMem, 0, 0, bm.bmWidth, bm.bmHeight,// : dc 
				SRCCOPY);
		}
	}


/**************************************************************
     Shape==Square-    
                    File Open   
                     .bmp-    
                      (, JPEG  PNG).
	  Color   .
**************************************************************/
	if(iShape == Square){
		//    CImage
		CImage m_Image;
		//      
		if(pWnd->strFileName.GetLength() > 0){ //    
			m_Image.Load(pWnd->strFileName);
			//   
			CRect rRect(iDisp, iDisp, iDisp + iWidth, iDisp + iHeight); 
			//    
			m_Image.Draw(dc->m_hDC, rRect);
		}
	}
}
